

## We recommend directly installing BAMMtools from 
##	www.bamm-project.org
##	or obtaining the package from BAMM
## 	rather than using the functions included here.

### You must install the BAMMtools package from source
##	to use this script (BAMMtools_1.0.tar.gz). Or obtain from CRAN.

##### Here we will go through a worked example from our paper
## using body shape evolution in skinks
##	You can replicate these results for body size
##	using the event data file "svl_event_data.txt"
##	and "div_event_data.txt"
##	If you use the speciation-extinction event data file (div_event_data.txt)
##	you must use the phylogeny with 216 taxa (skinktree_216.tre). Both the PC1 and
##	SVL analyses use the tree with 183 taxa.

library(BAMMtools);

v <- read.tree('skinktree183.txt');
v <- read.tree(text=write.tree(ladderize(v))); ### ladderize, fixing node ordering

# Read the BAMM output:
ed <- getEventData(v, 'pc1_event_data.txt', burnin=0.1, type = 'trait', nsamples=500);

## Inspect the posterior distribution of the number of shift events:
summary(ed);

## Generate a consensus phylorate plot:
plot.bammdata(ed, pal='temperature', lwd=1.5);

### compute marginal shift frequencies:
msc <- marginalShiftProbsTree(ed);

## plot a tree with branches scaled by marginal shift probabilities:
##	e.g,. Figure 9 in the paper
plot(msc, show.tip.label=F);
add.scale.bar(length=0.5); # scale bar: 

### Get the set of distinct shift configurations
ds <- distinctShiftConfigurations(ed, threshold=0.01);

# here is the ranked frequency distribution of all 
#	distinct shift configurations in the sample
plot(ds$frequency);  

ds$frequency; #gives this directly. 
# Note that just a few shift configurations are much more common than others
#	e.g., there are lots of rare shifts.

# what if we truly want to sample EVERY distinct shift configuration?
# If this is unclear, please see Figure S7 in the paper which explains 
#	why we do not normally want to count every single distinct shift configuration:
ds_all <- distinctShiftConfigurations(ed, threshold = 0.0001);
# Now we have 228 distinct configurations, but most are meaningless variations 
#	on the same theme, distinguished only by the presence of a single shifts with very 
#	low marginal probabilities

# Here's the frequency distribution for all:
plot(ds_all$frequency);
# but of course, as you could see from the marginal shift prob tree (msc)
#	only a few of these shifts have any reasonable probability. 
#	Most are effectively singletons.


## PLot the 9 most frequent distinct shift configurations
##	showing locations of the shifts and the corresponding phylorate plots:
quartz.options(height=10, width=10, dpi=72);
plot.new();
plot.bammshifts(sc = ds, ephy = ed);

### Compute clade-specific rates:
###	first, we need node numbers for Ctenotus and Lerista for MCC tree:
### Ctenotus = 189
### Lerista  = 270
#### both = 188
ct.node <- 189
le.node <- 270
both.node <- 188

rates.lerista <- getCladeRates(ed, node = le.node);
rates.ctenotus <- getCladeRates(ed, node = ct.node);
rates.background <- getCladeRates(ed, node = both.node, nodetype = 'exclude');
# this computes the rates for the sphenormophine clade but EXCLUDING ctenotus and lerista

# each of these objects (rates.ctenotus etc) is a list, containing the mean time-averaged rate 
# for the group for each sample in the posterior. hence, we can extract means and quantiles 
#	etc for these.

# compare rates:
quantile(rates.lerista$beta, c(0.05, 0.5, 0.95));
quantile(rates.ctenotus$beta, c(0.05, 0.5, 0.95));
quantile(rates.background$beta, c(0.05, 0.5, 0.95));

## Ctenotus rates are massively lower than the other groups

## Here is a rate-through-time analysis:
start.time <- 0.1;
end.time <- 0.99 * max(branching.times(v));
rtt.ctenotus <- getRateThroughTimeMatrix(ed, node=ct.node, start.time = start.time, end.time = end.time);
rtt.lerista <- getRateThroughTimeMatrix(ed, node=le.node, start.time = start.time, end.time = end.time);
rtt.background <- getRateThroughTimeMatrix(ed, node=both.node, start.time = start.time, end.time = end.time, nodetype = 'exclude');

# these objects include the rate at each time point for each sample in the posterior:
names(rtt.ctenotus)

# you can easily extract confidence intervals, quantiles etc for any time point 
# to generate a plot like Fig. 7 in the paper

## Now we'll get mean rate-through-time curves:
ct.mean <- colMeans(rtt.ctenotus$beta);
le.mean <- colMeans(rtt.lerista$beta);
other.mean <- colMeans(rtt.background$beta);


### here is a simple plot of mean rates for all 3 groups:
quartz.options(height=6, width=6);
plot.new();
par(oma=c(1,1,1,1));
par(mar=c(6,6,1,1));
plot.window(xlim=c(0, 25), ylim=c(0,3));
lines(x=rtt.background$times, y=other.mean, lwd=3, col='black');
lines(x=rtt.lerista$times, y=le.mean, lwd=3, col='red');
lines(x=rtt.ctenotus$times, y=ct.mean, lwd=3, col='blue');

axis(1);
axis(2, las=1);
mtext('Time from start of radiation (my)', side=1, line=3.5, cex= 1.3)
mtext('Phenotypic rate (shape: PC1)', side=2, line=3.5, cex= 1.3)

#### 

